from pathlib import Path
from typing import TYPE_CHECKING, Callable
from uuid import uuid4

try:
    import yappi
except ImportError as import_error:  # pragma: no cover
    msg = (
        "Profiling is not available without yappi installed. "
        "Either install it or disable profiling."
    )
    raise ImportError(msg) from import_error

if TYPE_CHECKING:
    from fastapi import FastAPI, Request, Response


def set_yappi_profiler(app: "FastAPI") -> None:
    @app.middleware("http")
    async def profile_process_time(
        request: "Request",
        call_next: Callable[["Request"], "Response"],
    ) -> "Response":
        yappi.clear_stats()
        yappi.set_clock_type("cpu")
        yappi.start()

        response = await call_next(request)

        yappi.stop()

        folder_location = Path(__file__).parents[2] / "profiling"
        folder_location.mkdir(parents=True, exist_ok=True)

        profiling_log_id = uuid4()

        yappi.convert2pstats(yappi.get_func_stats()).dump_stats(
            f"{folder_location}/{profiling_log_id}.pstats",
        )

        response.headers["profiling-log-id"] = str(profiling_log_id)

        return response



from typing import TYPE_CHECKING

from pydantic import UUID4, BaseModel, PositiveInt

from yak_server.helpers.language import Lang, get_language_description

from .groups import GroupIn, GroupOut
from .phases import PhaseOut
from .teams import FlagOut, TeamIn, TeamModifyScoreBetIn, TeamWithScoreOut

if TYPE_CHECKING:
    from yak_server.database.models import ScoreBetModel


class ScoreBetIn(BaseModel):
    index: PositiveInt
    team1: TeamIn
    team2: TeamIn
    group: GroupIn


class ScoreBetOut(BaseModel):
    id: UUID4
    locked: bool
    team1: TeamWithScoreOut
    team2: TeamWithScoreOut

    @classmethod
    def from_instance(
        cls,
        score_bet: "ScoreBetModel",
        *,
        locked: bool,
        lang: Lang,
    ) -> "ScoreBetOut":
        return cls(
            id=score_bet.id,
            locked=locked,
            team1=TeamWithScoreOut(
                id=score_bet.match.team1.id,
                code=score_bet.match.team1.code,
                description=get_language_description(score_bet.match.team1, lang),
                score=score_bet.score1,
                flag=FlagOut(url=score_bet.match.team1.flag_url),
            ),
            team2=TeamWithScoreOut(
                id=score_bet.match.team2.id,
                code=score_bet.match.team2.code,
                description=get_language_description(score_bet.match.team2, lang),
                score=score_bet.score2,
                flag=FlagOut(url=score_bet.match.team2.flag_url),
            ),
        )


class Group(BaseModel):
    id: UUID4


class ScoreBetWithGroupIdOut(BaseModel):
    id: UUID4
    locked: bool
    group: Group
    team1: TeamWithScoreOut
    team2: TeamWithScoreOut

    @classmethod
    def from_instance(
        cls,
        score_bet: "ScoreBetModel",
        *,
        locked: bool,
        lang: Lang,
    ) -> "ScoreBetWithGroupIdOut":
        description_team1 = get_language_description(score_bet.match.team1, lang)
        description_team2 = get_language_description(score_bet.match.team2, lang)

        return cls(
            id=score_bet.id,
            locked=locked,
            group=Group(id=score_bet.match.group_id),
            team1=TeamWithScoreOut(
                id=score_bet.match.team1.id,
                code=score_bet.match.team1.code,
                description=description_team1,
                score=score_bet.score1,
                flag=FlagOut(url=score_bet.match.team1.flag_url),
            ),
            team2=TeamWithScoreOut(
                id=score_bet.match.team2.id,
                code=score_bet.match.team2.code,
                description=description_team2,
                score=score_bet.score2,
                flag=FlagOut(url=score_bet.match.team2.flag_url),
            ),
        )


class ScoreBetResponse(BaseModel):
    phase: PhaseOut
    group: GroupOut
    score_bet: ScoreBetOut


class ModifyScoreBetIn(BaseModel):
    team1: TeamModifyScoreBetIn
    team2: TeamModifyScoreBetIn



from datetime import datetime
from functools import lru_cache
from typing import Iterator, List
from uuid import UUID

from pydantic import BaseModel, RootModel
from pydantic_settings import BaseSettings, SettingsConfigDict


class RuleContainer(BaseModel):
    id: UUID
    config: dict


class Rules(RootModel):
    root: List[RuleContainer]

    def __iter__(self) -> Iterator[RuleContainer]:
        return iter(self.root)

    def append(self, rule: RuleContainer) -> None:
        self.root.append(rule)


class Settings(BaseSettings):
    jwt_secret_key: str
    jwt_expiration_time: int
    competition: str
    lock_datetime: datetime
    base_correct_result: int
    multiplying_factor_correct_result: int
    base_correct_score: int
    multiplying_factor_correct_score: int
    team_qualified: int
    first_team_qualified: int
    data_folder: str
    rules: Rules

    model_config = SettingsConfigDict(env_file=".env", env_file_encoding="utf-8", extra="allow")


@lru_cache(maxsize=None)
def get_settings() -> Settings:
    return Settings()
